@@ -1,5 +1,6 @@
 package misc;
 use File::Copy qw/copy/;
+use Path::Class qw( tempdir );
 require Test::More;
 
 sub import {
@@ -12,10 +13,10 @@ use strict;
 
 sub sysok {
     my $cmd = shift;
-    my $log = File::Temp->new();
+    my $log = tempdir( CLEANUP => 1)->file("log.txt");
     ok system($cmd . " > $log 2>&1")==0, "$cmd" or do {
         diag "$cmd failed : $? ".(${^CHILD_ERROR_NATIVE} || '');
-        diag $_ for $log->getlines;
+        diag $_ for $log->slurp;
     };
 }
 
